@@ -395,22 +395,6 @@ bool Core::startListening() {
         );
       } else {
         switch(addr.protocol()) {
-          case QAbstractSocket::IPv4Protocol:
-            if(_server.listen(addr, port)) {
-              quInfo() << qPrintable(
-                tr("Listening for GUI clients on IPv4 %1 port %2 using protocol version %3")
-                  .arg(addr.toString())
-                  .arg(_server.serverPort())
-                  .arg(Quassel::buildInfo().protocolVersion)
-              );
-              success = true;
-            } else
-              quWarning() << qPrintable(
-                tr("Could not open IPv4 interface %1:%2: %3")
-                  .arg(addr.toString())
-                  .arg(port)
-                  .arg(_server.errorString()));
-            break;
           case QAbstractSocket::IPv6Protocol:
             if(_v6server.listen(addr, port)) {
               quInfo() << qPrintable(
@@ -420,15 +404,30 @@ bool Core::startListening() {
                   .arg(Quassel::buildInfo().protocolVersion)
               );
               success = true;
-            } else {
-              // if v4 succeeded on Any, the port will be already in use - don't display the error then
-              // FIXME: handle this more sanely, make sure we can listen to both v4 and v6 by default!
-              if(!success || _v6server.serverError() != QAbstractSocket::AddressInUseError)
-                quWarning() << qPrintable(
-                  tr("Could not open IPv6 interface %1:%2: %3")
+            } else
+              quWarning() << qPrintable(
+                tr("Could not open IPv6 interface %1:%2: %3")
                   .arg(addr.toString())
                   .arg(port)
                   .arg(_v6server.errorString()));
+            break;
+          case QAbstractSocket::IPv4Protocol:
+            if(_server.listen(addr, port)) {
+              quInfo() << qPrintable(
+                tr("Listening for GUI clients on IPv4 %1 port %2 using protocol version %3")
+                  .arg(addr.toString())
+                  .arg(_server.serverPort())
+                  .arg(Quassel::buildInfo().protocolVersion)
+              );
+              success = true;
+            } else {
+              // if v6 succeeded on Any, the port will be already in use - don't display the error then
+              if(!success || _server.serverError() != QAbstractSocket::AddressInUseError)
+                quWarning() << qPrintable(
+                  tr("Could not open IPv4 interface %1:%2: %3")
+                  .arg(addr.toString())
+                  .arg(port)
+                  .arg(_server.errorString()));
             }
             break;
           default:
